# ----------------------------------------------------------------------------------------
# 					SIMULATION PARAMETERS FOR THE PIC-CODE SMILEI
# ----------------------------------------------------------------------------------------

#					CONFIGURATION FOR UMEA LASER


from math import pi, sqrt, exp, log

# lambda_l = 0.75 micron
# FWHM_I = 0.84932 w0

proton_mass = 1836.2	# in me


l0 = 2. * pi             # laser wavelength [in code units]
t0 = l0                  # optical cycle, lambda_l/c = 2.5 fs
Lsim = [32.*l0, 32.*l0]  # length of the simulation, [24, 24] um
Tsim = 26.*t0             # duration of the simulation, 100 fs
resx = 25.               # nb of cells in one laser wavelength
rest = resx*sqrt(2.)/0.95 # nb of timesteps in one optical cycle 
xf = 12.0*l0		# focus distance from xmin, 9 um
w0 = 1.35891*l0              # waist 1.0192 um, i.e. FWHM of Intensity is 1.2 um
a0 = 6.4125                # amplitude, i.e. I = 1.0e20W/cm2
FWHMtinI = 2.0*t0		# 5 fs
pulse_start = FWHMtinI*2
n0max = 3.0  # electron density (code units => 1 = plasma critical density)
rball = 0.4*l0		# 0.3 um

def time_envelope(t):
#    return 1 # plane wave
    return exp(-2.*log(2.)*(t-xf-pulse_start)**2/FWHMtinI**2)
    
def carrier(t):
    return np.cos(t)

def space_envelope(y):
    return np.exp(-(y-Lsim[1]/2.0)**2.0/w0**2.0) #gauss
    
def By_profile(y,t):
    return 0

def Bz_profile(y,t):
    return a0*space_envelope(y) * time_envelope(t) * carrier(t)

def n0_electron(x,y):
    if sqrt((x-xf)**2 +(y-Lsim[1]/2.0)**2) < rball:
	    return n0max
    else:
	    return 0

def n0_carbon(x,y):
	return n0_electron(x,y)/8.
    
def n0_proton(x,y):
        return n0_electron(x,y)/4.


Main(
    geometry = "2Dcartesian",
    
    interpolation_order = 4 ,
    
    cell_length = [l0/resx,l0/resx],
    grid_length  = Lsim,
    
    number_of_patches = [ 8, 8 ],
    
    timestep = t0/rest,
    simulation_time = Tsim,
     
   EM_boundary_conditions = [
        ['silver-muller'],
        ['silver-muller'],
    ],
    
    random_seed = smilei_mpi_rank
)

LaserOffset(
    box_side               = "xmin",
    space_time_profile     = [ By_profile, Bz_profile ],
    offset                 = xf,
    #extra_envelope          = tconstant(),
    keep_n_strongest_modes = 256,
    angle = 0
)


Species( 
    name = "electron",
    position_initialization = "random",
    momentum_initialization = "mj",
    particles_per_cell = 10,
#    c_part_max = 1.0,
    mass = 1.0,
    charge = -1.0,
    number_density = n0_electron,
    temperature = [0.001],
    mean_velocity = [0.0, 0.0, 0.0],
    pusher = "boris",    
    time_frozen = 0.0,
    boundary_conditions = [
        ["remove", "remove"],
        ["remove", "remove"],
    ],
)

Species(
    name = "carbon",
    position_initialization = "random",
    momentum_initialization = "mj",
    particles_per_cell = 10,
#    c_part_max = 1.0,
    mass = proton_mass*12.0,
    charge = 6.0,
    number_density = n0_carbon,
    temperature = [0.001],
    mean_velocity = [0.0, 0.0, 0.0],
    pusher = "boris",
    time_frozen = 0.0,
    boundary_conditions = [
        ["remove", "remove"],
        ["remove", "remove"],
    ],
)

Species(
    name = "proton",
    position_initialization = "random",
    momentum_initialization = "mj",
    particles_per_cell = 10,
#    c_part_max = 1.0,
    mass = proton_mass,
    charge = 1.0,
    number_density = n0_proton,
    temperature = [0.001],
    mean_velocity = [0.0, 0.0, 0.0],
    pusher = "boris",
    time_frozen = 0.0,
    boundary_conditions = [
        ["remove", "remove"],
        ["remove", "remove"],
    ],
)

DiagScalar(
    every = 2*rest
)

DiagFields(
    every = 2*rest,
    fields = ['Ex','Ey','Ez','Bx','By','Bz','Rho_electron','Rho_proton','Rho_carbon']
)
